/**
 * SALSA/World Wide Computer Project
 *
 * Location Server - Universal Actor Name Daemon
 * Provides the location (Universal Actor Location) of an actor running on
 * the World Wide Computer from its name (Universal Actor Name).
 *
 * By Gregory Haik and Carlos Varela.  v0.1  June, 1999
 *
 */
package wwc.naming.locationServer;

import java.net.*;
import java.io.*;
import java.util.Hashtable;
import wwc.naming.*;

/**
 * Contains the main method of the UAN daemon.
 *
 * @version %I%, %G%
 * @author Gregory Haik, Carlos Varela
 */
public class UANDaemon {
	static int portNumber = 3030;							// The default port number is set to 3030.
	static ServerSocket serverSocket = null;				// The server will listen via this socket.
	static String protocolVersion = UANProtocol.VERSION;	// This server implements the very first version of UAN protocol.
	static String serverVersion = new String("uand/0.1");	// Location server version.
	static Hashtable locationMap;							// Mapping from UAN's to UAL's.
//???Static?
	static String response;									// Used for communications.
//???Static?
	static BufferedReader in;								// Used for communications.
//???Static?
	static PrintWriter out;									// Used for communications.
	static private boolean flag = true;							// Flag variable for while loop.

	public static void main(String[] argv) {
		for (int x=0; x<argv.length; x++) {
			if (argv[x].equals("-h"))
				helpMessage();
			else if (argv[x].equals("-v"))
				printServerVersion();
			else if (argv[x].equals("-p")) {
				x++;
				try {
					portNumber = Integer.parseInt(argv[x]);
				} catch (Exception e) {
					helpMessage("Invalid port number: "+argv[x]);
				}
			}
			else
				helpMessage("illegal option");
		}
/*
		if (argv.length > 0) 
			if (argv.length == 1) 
				if (argv[0].equals("-h")) 
					UANDaemon.helpMessage();
				else if (argv[0].equals("-v"))
					UANDaemon.printServerVersion();
				else
					UANDaemon.helpMessage("illegal option");
			else if  (argv.length != 2)
				UANDaemon.helpMessage("too many arguments");
			else if (!(argv[0].equals("-p")))
				UANDaemon.helpMessage("wrong options");
			else try { 
					portNumber = Integer.parseInt(argv[1]);
				} catch (Exception e) {
					UANDaemon.helpMessage("invalid port number : " + argv[1]);
				}
*/
		try {
			serverSocket = new ServerSocket(portNumber);
		} catch (IOException e) {
			System.out.println("Could not listen on port: "+portNumber+", "+e);
			System.exit(1);
		}
		locationMap = new Hashtable();
		while (flag) {
			Socket clientSocket = null;
			try {
				clientSocket = serverSocket.accept();
				in = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
				out = new PrintWriter(clientSocket.getOutputStream(), true);
			} catch (IOException e) {
				System.out.println("Accept failed: "+portNumber+", "+e);
				System.exit(1);
			}
			try {
				try {
					LocationRequest request = new LocationRequest(in, locationMap);  
					response = request.process();
				} catch (BadUANPRequestException e) {
					response = new String(UANProtocol.VERSION 
						+ " "
						+ UANProtocol.BAD_REQ_STATUS_CODE 
						+ " "
						+ UANProtocol.BAD_REQ_STATUS_STR
						+ "\n");
				}
				System.out.println("Response : "+response);
				out.println(response);
				in.close();
				out.close();
				clientSocket.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	static void printServerVersion() {
		System.out.println("Universal Actor Name Daemon.");
		System.out.println(serverVersion);
		System.exit(0);
	}

	static void helpMessage(String s) {
		System.out.println(s);
		UANDaemon.printHelpMessage();
		System.exit(1);
	}

	static void helpMessage() {
		UANDaemon.printHelpMessage();
		System.exit(0);
	}

	static void printHelpMessage() {
		System.out.println("usage:");
		System.out.println("  java ...UANDaemon");
		System.out.println("  java ...UANDaemon -h");
		System.out.println("  java ...UANDaemon -v");
		System.out.println("  java ...UANDaemon -p portNumber");
		System.out.println("options:");
		System.out.println("  -h : print this message");
		System.out.println("  -v : print version");
		System.out.println("  -p portNumber : set the listening port to portNumber");
		System.out.println("                  default port number is 3030");
	}
}







